@@ -288,12 +288,7 @@ sub end {
   return $self->{end};
 }
 
-sub DESTROY {
-  my $self = shift;
-  if ($self->{dirty} && $self->{write}) {
-    $self->_writeoptions($self->{options});
-  }
-}
+sub DESTROY { shift->write }
 
 sub erase {
   my $self = shift;
@@ -344,6 +339,15 @@ sub smoothing {
   $d;
 }
 
+sub write {
+  my $self = shift;
+  if ($self->{dirty} && $self->{write}) {
+    $self->_writeoptions($self->{options});
+    undef $self->{dirty};
+    $self->fh->flush;
+  }
+}
+
 sub _readoptions {
   my $self = shift;
   my $fh = $self->fh;
